/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.analyzer;

public interface IFact {

	public enum Connective { OR, AND, IFF, WON }
	
	public enum FactType { QUERY, COMMAND, STATEMENT }
	
	/**
	 * @return
	 */
	public FactType getFactType();
	
	/**gets the connective in a composite fact
	 * @return the connective in a composite fact
	 * @throws AnalysisException when this method is called on a non-simple fact
	 */
	public Connective getConnective() throws AnalysisException;
	
	public boolean getNegLeft() throws AnalysisException;
	public boolean getNegRight() throws AnalysisException;
	
	public IFact getLeft() throws AnalysisException;
	public IFact getRight() throws AnalysisException;
	
	/**is this fact true, or false?
	 * @return
	 */
	public boolean isTrue() throws AnalysisException;
	
	/**get subject n in the simple fact
	 * @param n the number of the subject desired in the fact, starting with 0 up to 4
	 * @return that subject in the fact
	 * @throws AnalysisException when this method is called on a non-composite fact
	 */
	public ISubject getSubject(int n) throws AnalysisException;
	
	/**gets the relation in a fact
	 * @return the relation in a fact
	 * @throws AnalysisException when this method is called on a non-simple fact
	 */
	public IRelation getRelation() throws AnalysisException;
	
	/**Find out if this is a composite fact (using connectives) or a simple fact
	 * @return true if this fact is composite, false otherwise
	 */
	public boolean isComposite();
	
	/**Find out if this fact contains a query
	 * @return true if this fact is a query or contains a query
	 */
	public boolean isQuery();
	
	/**
	 * Find out if this fact is a yes/no query.
	 * @return true if this is a yes/no query
	 */
	public boolean isConfirmQuery();
	
	public IFact substitute(ISubject id, ISubject s);
}
