/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.analyzer;

import lojban.parser.GenericParseNode;

/**
 * @author mike
 * Analyzes the nodes which define a Selbri and resolve to a relation.
 */
class RelationAnalysis extends AbstractRecursiveAnalysis {

	private String relation = null;
	private boolean negate = false;
	
	@SuppressWarnings("unused")
	private TextAnalysis ta = null;
	
	/** assignment of probridi set (cei clause) */
	private boolean assignment = false;
	
	/** probridi assignment */
	private String probridiAssignment = null;
	private String probridiInvocation = null;
	
	public RelationAnalysis(TextAnalysis ta) {
		this.ta = ta;
	}
	
	/**
	 * Copy relationanalysis and optionally flip truth value.
	 * THIS IS NOT FOR PARSING
	 * @param ra
	 * @param prosumtiNegate if true, flip the truth value
	 */
	public RelationAnalysis(RelationAnalysis ra, boolean prosumtiNegate) {
		relation = ra.relation;
		negate = ra.negate ^ prosumtiNegate;
		ta = ra.ta;
		assignment = ra.assignment;
		probridiAssignment = ra.probridiAssignment;
		probridiInvocation = ra.probridiInvocation;
	}

	/* (non-Javadoc)
	 * @see lojban.analyzer.AbstractRecursiveAnalysis#getDebugName()
	 */
	public String getDebugName() {
		return "Relation";
	}
	
	/**
	 * @return true if sentence is negated
	 */
	public boolean isNegated() {
		return negate;
	}
		
	/**
	 * Resolve to a relation object.
	 * @param resolver resolver object
	 * @return a relation
	 */
	public IRelation resolve(IResolver resolver) {
		return resolver.getRelationByName(relation);
	}
	
	public void processSelbri(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processSelbri1(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processSelbri2(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processSelbri3(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processSelbri4(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processSelbri5(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processSelbri6(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processTanruunit(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processTanruunit1(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processTanruunit2(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processBrivlaclause(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processBrivlapre(GenericParseNode n) {
		processChildren(n);
	}
	
	public void processBrivla(GenericParseNode n) {
		processChildren(n);
	}
	
	/**
	 * pro-bridi assignment has occurred
	 * @param n
	 */
	public void processCeiclause(GenericParseNode n) {
		assignment = true;
	}
	
	/**
	 * Detect negation.
	 * @param n
	 */
	public void processNaclause(GenericParseNode n) {
		this.negate = true;
	}
	
	/**
	 * Collect gismu to make a string to identify this relation.
	 * @param n
	 */
	public void processGismu(GenericParseNode n) {
		if(!assignment) {			// no assignment seen yet
			
			// add to gismu
			if(relation == null)
				relation = n.getContent();
			else
				relation += " " + n.getContent();
			
			// this is a probridi invocation
			if(n.getContent().length() == 5 && n.getContent().startsWith("brod")) {
				probridiInvocation = n.getContent();
			}
						
		} else {					// this is now an assignment
			probridiAssignment = n.getContent();
		}
	}
	
	/**
	 * This is a an anaphoric probridi (go'i)
	 * @param n
	 */
	public void processGohaclause(GenericParseNode n) {
		probridiInvocation = n.getText().trim();
	}

	/**
	 * @return if this relation assigns a probridi, return it
	 */
	public String getAssignment() {
		return probridiAssignment;
	}
	
	/**
	 * @return if this invokes assigns a probridi, return it
	 */
	public String getInvocation() {
		return probridiInvocation;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return relation;
	}
}
