/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.gui;

import java.util.Vector;

import com.sun.speech.freetts.Voice;
import com.sun.speech.freetts.VoiceManager;

import lojban.agent.IConversationListener;
import lojban.agent.LojbanContext;
import lojban.agent.LojbanPlugin;

public class TextToSpeechPlugin extends Thread implements
		IConversationListener, LojbanPlugin {

	private String voice = new String("lojban");

	private Vector<String> speakables = new Vector<String>();

	private LojbanContext lc;

	Voice helloVoice = null;

	public void input(String s) {
		synchronized(speakables) {
			speakables.add(s);
			speakables.notifyAll();
		}
	}

	public void output(String s) {
		synchronized(speakables) {
			speakables.add(s);
			speakables.notifyAll();
		}
	}

	public void error(String s) {
		
	}

	private void SpeakString(String utterance) {

		if (helloVoice == null) {
			System.err.println("Cannot find a voice named " + voice
					+ ".  Please specify a different voice.");
			System.exit(1);
		}

		helloVoice.speak(utterance);

	}

	public void unload() {
		helloVoice.deallocate();
		lc.removeConversationListener(this);
	}

	public void load(LojbanContext lc) {

		this.lc = lc;
		VoiceManager voiceManager = VoiceManager.getInstance();
		helloVoice = voiceManager.getVoice(voice);

		if (helloVoice == null) {
			System.err.println("Cannot find a voice named " + voice
					+ ".  Please specify a different voice.");
			System.exit(1);
		}

		helloVoice.allocate();

		lc.addConversationListener(this);
		
		if(!this.isAlive())
			this.start();
	}
	
	public String getPluginName() {
		return "Text-to-speech";
	}
	
	public void status(String s) {
		
	}

	public void run() {
		
		while(true) {
			
			String nextString = null;
			
			synchronized(speakables) {
				try {
					while(speakables.isEmpty())
						speakables.wait();
					nextString = speakables.firstElement();
					speakables.removeElementAt(0);
				} catch(Exception e) {
					e.printStackTrace();
				}
			}
			
			SpeakString(nextString);
		}
			
	}
}
