/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.kb.jtp;

import lojban.analyzer.IFact;
import lojban.analyzer.ISubject;

public class JTPConcreteSubject extends JTPSubject {

	private JTPFact query;
	private JTPLiteral q;
	
	public JTPConcreteSubject(JTPFact query, JTPLiteral q) {
		this.q = q;
		this.query = query;
		q.setConcrete(true);
	}

	public String toQuery() {
		return query.toInnerQuery(false);
	}
	
	public String toString() {
		return q.toString();
	}
	
	public JTPLateSubject[] getLateSubjects() {
		return query.getLateSubjects();
	}
	
	public JTPConcreteSubject[] getConcreteSubjects() {
		return query.getConcreteSubjects();
	}
	
	public String debugString() {
		return q + " in " + query.toQuery();
	}


	/* (non-Javadoc)
	 * @see lojban.kb.jtp2.JTPSubject#matchesId(java.lang.String)
	 */
	public boolean matchesId(JTPSubject id) {
		return q.matchesId(id);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object o){
		if(o instanceof JTPLiteral){
			return q.equals(o);
		}else if(o instanceof JTPLateSubject){
			return(this.query.equals(((JTPConcreteSubject) o).query) && this.q.equals(((JTPConcreteSubject) o).q));
		}else
			return false;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode(){
		return(query.hashCode() ^ q.hashCode());
	}
	
	public boolean isRestricted() {
		return true;
	}
	
	public IFact getRestricting() {
		return query;
	}
	
	public ISubject getRestrictingIdentifier() {
		return q;
	}
}
