/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.kb.jtp;

public class JTPLiteral extends JTPSubject {

	String literal;
	boolean concrete = false;
	boolean prosumti = false;
	
	public JTPLiteral(String s) {
		literal = s;
		concrete = true;
	}
	
	public JTPLiteral(String s, boolean b) {
		concrete = b;
		literal = s;
	}
	
	public String toString() {
		if(concrete) {
			if(literal.startsWith("q"))
				return "s" + literal.substring(1);
			else {
				if(isLiteral()) {
					if(literal.startsWith("\""))
						return literal;
					else
						return "\"" + literal + "\"";
				} else {
					return literal;
				}
			}
		} else
			return "?" + literal;
	}

	/* (non-Javadoc)
	 * @see lojban.kb.jtp2.JTPSubject#getLateSubjects()
	 */
	public JTPLateSubject[] getLateSubjects() {
		return new JTPLateSubject[]{};
	}

	public void setConcrete(boolean b) {
		concrete = b;
	}
	
	public boolean equals(Object o){
		if(o instanceof JTPLiteral){
			if(this.concrete == ((JTPLiteral) o).concrete && this.literal.equals(((JTPLiteral) o).literal))
				return true;
			else
				return false;
		}else{
			return false;
		}
	}
	
	public int hashCode(){
		return literal.hashCode() + (concrete?1:0);
	}

	/* (non-Javadoc)
	 * @see lojban.analyzer.ISubject#isLiteral()
	 */
	public boolean isLiteral() {
		return !Character.isDigit(literal.charAt(literal.length()-1)); 
	}
	
	/* (non-Javadoc)
	 * @see lojban.analyzer.ISubject#isIdentifier()
	 */
	public boolean isIdentifier() {
		return Character.isDigit(literal.charAt(literal.length()-1));
	}

	/* (non-Javadoc)
	 * @see lojban.kb.jtp2.JTPSubject#matchesId(java.lang.String)
	 */
	public boolean matchesId(JTPSubject id) {
		return toString().equals(id.toString());
	}
	
	public static JTPLiteral getProsumti(String s) {
		JTPLiteral l = new JTPLiteral(s);
		l.prosumti = true;
		return l;
	}
	
	public boolean isProsumti() {
		return prosumti;
	}
}
