/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.kb.jtp;

import java.util.LinkedList;

import lojban.analyzer.AnalysisException;
import lojban.analyzer.IRelation;
import lojban.analyzer.ISubject;

public class JTPSimpleFact extends JTPFact {

	// count dontcare identifiers for uniqueness
	private static int uniquecount = 0;
	
	// true if this is a yes/no question
	private boolean confirm = false;
	
	private boolean truth;
	private JTPSubject x[] = new JTPSubject[5];
	private JTPRelation r;
		
	public JTPSimpleFact(boolean isTrue, JTPRelation r, JTPSubject s1, JTPSubject s2, JTPSubject s3, JTPSubject s4, JTPSubject s5) {
		x[0] = s1;
		x[1] = s2;
		x[2] = s3;
		x[3] = s4;
		x[4] = s5;
		this.r = r;
		truth = isTrue;
	}
	
	public JTPSimpleFact(boolean isTrue, JTPRelation r, JTPSubject s1, JTPSubject s2, JTPSubject s3, JTPSubject s4, JTPSubject s5, boolean confirmQuery) {
		x[0] = s1;
		x[1] = s2;
		x[2] = s3;
		x[3] = s4;
		x[4] = s5;
		this.r = r;
		truth = isTrue;
		confirm = confirmQuery;
	}
	
	public boolean equals(Object o){
		if(o instanceof JTPSimpleFact){
			if(this.r.equals(((JTPSimpleFact) o).r) && this.truth == ((JTPSimpleFact) o).truth){
				for(int i=0;i<5;i++){
					if( (this.x[i]==null && ((JTPSimpleFact) o).x[i]!=null) || (this.x[i]!=null && ((JTPSimpleFact) o).x[i]==null))
						return false;
					if(this.x[i]!=null && ! this.x[i].equals(((JTPSimpleFact) o).x[i]))
						return false;
				}
				return true;
			}else
				return false;
		}else{
			return false;
		}
	}
	
	public int hashCode(){
		int ret=r.hashCode() + (truth?1:0);
		for(int i=0;i<5;i++)
			if(x[i]!=null) ret^=x[i].hashCode();
		return ret;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return toInnerQuery(false);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toInnerQuery(boolean soft) {
		boolean query = isQuery() | isConfirmQuery() | soft;
		
		StringBuffer sb = new StringBuffer();
		if(!truth)
			sb.append("(factnot ");
		else
			sb.append("(fact ");
		sb.append(r);
		for(int i = 0; i < 5; i++) {
			if(x[i] != null)
				sb.append(" " + x[i]);
			else {
				if(query)
					sb.append(" ?dontcare" + (++uniquecount));
				else
					sb.append(" \"zohe0\"");
			}
		}
		sb.append(")");
		//if(!truth)
		//	sb.append(")");
		return sb.toString();
	}
	
	public boolean isQuery() {
		if(r.toString().startsWith("?"))
			return true;
		for(int i = 0; i < 5; i++)
			if(x[i] != null && x[i].toString().startsWith("?ask"))
				return true;
		return false;
	}
	
	public boolean isConfirmQuery() {
		return confirm;
	}

	/* (non-Javadoc)
	 * @see lojban.kb.jtp2.JTPFact#getLateSubjects()
	 */
	public JTPLateSubject[] getLateSubjects() {
		LinkedList<JTPLateSubject> subjs = new LinkedList<JTPLateSubject>();
		for(JTPSubject s : x)
			if(s instanceof JTPLateSubject && !subjs.contains(s)) {
				subjs.add((JTPLateSubject)s);
				for(JTPLateSubject s1 : ((JTPLateSubject)s).getLateSubjects())
					subjs.add(s1);
			}
		return subjs.toArray(new JTPLateSubject[]{});
	}

	/* (non-Javadoc)
	 * @see lojban.kb.jtp2.JTPFact#getConcreteSubjects()
	 */
	public JTPConcreteSubject[] getConcreteSubjects() {
		LinkedList<JTPConcreteSubject> subjs = new LinkedList<JTPConcreteSubject>();
		for(JTPSubject s : x)
			if(s instanceof JTPConcreteSubject && !subjs.contains(s)) {
				subjs.add((JTPConcreteSubject)s);
				for(JTPConcreteSubject s1 : ((JTPConcreteSubject)s).getConcreteSubjects())
					subjs.add(s1);
			}
		return subjs.toArray(new JTPConcreteSubject[]{});
	}


	/* (non-Javadoc)
	 * @see lojban.kb.jtp2.JTPFact#substitute(lojban.kb.jtp2.JTPSubject, lojban.kb.jtp2.JTPSubject)
	 */
	public JTPFact substitute(JTPSubject id, JTPSubject s) {
		JTPSimpleFact newFact = new JTPSimpleFact(truth, r, x[0], x[1], x[2], x[3], x[4]);
		for(int i = 0; i < 5; i++) {
			if(newFact.x[i] != null && newFact.x[i].matchesId(id))
				newFact.x[i] = s;
		}
		return newFact;
	}

	/* (non-Javadoc)
	 * @see lojban.analyzer.IFact#getConnective()
	 */
	public Connective getConnective() throws AnalysisException {
		throw new AnalysisException("Cannot get the connective of a simple fact");
	}

	/* (non-Javadoc)
	 * @see lojban.analyzer.IFact#getSubject(int)
	 */
	public ISubject getSubject(int n) throws AnalysisException {
		if(n<0||n>4)
			throw new AnalysisException("Only subject 0 through 4, inclusive, are parts of a fact");
		else
			return x[n];
	}

	/* (non-Javadoc)
	 * @see lojban.analyzer.IFact#getRelation()
	 */
	public IRelation getRelation() throws AnalysisException {
		return r;
	}

	/* (non-Javadoc)
	 * @see lojban.analyzer.IFact#isComposite()
	 */
	public boolean isComposite() {
		return false;
	}

	public boolean isTrue() throws AnalysisException{
		return truth;
	}
}
