/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.parser;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.TreeMap;

public class EnglishDictionary extends AbstractDictionary {
	
	/** string table */
	private TreeMap<String, String> wordmap = new TreeMap<String, String>();
	
	/**
	 * Initialize wordmap.
	 */
	public EnglishDictionary() {
		wordmap.put("le", "the");
		wordmap.put("la", "a");
		wordmap.put("pe", "associated with");
		wordmap.put("po", "uniquely associated with");
		wordmap.put("noi", "incidentally");
		wordmap.put("poi", "which");
		wordmap.put("?", "(it?)");
		wordmap.put("ma", "(who/what/where...?)");
		wordmap.put("ko'a", "it");
		wordmap.put("ke'a", "it");
		wordmap.put("mi", "I/me");
		wordmap.put("do", "you");
		wordmap.put("mo", "(is/does what?)");
		wordmap.put("motag", "(to/from/for/by/with...:)");
		wordmap.put("xu", "(true/false?)");
		
		wordmap.put("na", "not");
		wordmap.put("a", "or");
		wordmap.put("e", "and");
		wordmap.put("o", "if-and-only-if");
		wordmap.put("u", "whether-or-not");
	}
	
	public String getDictionaryFile() {
		return "gismu.txt";
	}

	public String[] getSeparators() {
		return new String[]{" to", " from", " of", " with", " for", " used by", " by", " about", " in"};
	}

	public String getWord(String s) {
		if(wordmap.containsKey(s))
			return wordmap.get(s);
		else
			return "<" + s + ">";
	}
	
	public int getSkip() {
		return 2;
	}

	public int getNounStart() {
		return 20;
	}

	public int getNounEnd() {
		return 41;
	}
	
	public int getEntryStart() {
		return 62;
	}

	public int getEntryEnd() {
		return 159;
	}

	public static void main(String args[]) throws Exception {
		
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		
		EnglishDictionary ed = new EnglishDictionary();
		
		String line = null;
		while((line = br.readLine()) != null) {
		
			String gismu = line.trim();
			
			System.out.println(gismu.toUpperCase());
			System.out.println(" x1 " + ed.getVerb(gismu) + "...");
			for(int i = 2; i <= 5; i++) {
				String s = ed.getSubjectTag(gismu, i);
				if(!s.startsWith("<")) {
					System.out.println(" " + (s + " x" + i).trim());
				}
			}
		}
	}
}
