/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.parser;

import java.util.Vector;

/**
 * @author mike
 * Break a lojban utterance up into a series of sentences/clauses that resemble how a speaker
 * might pause while saying them (after the completion of each clause/sentence).  For use mainly
 * by TTS.
 */
public class SentenceTokenizer {

	public static String markSentences(String lojban) {
		
		synchronized(GenericParseNode.class) {
		
		Vector<GenericParseNode> gnp = null;
		try {
			gnp = GenericParseNode.parse(lojban);
		} catch(Exception e) {
			System.out.println("parse error on markSentences(): " + lojban);
			return lojban.replace(".", " ");
		}
		
		StringBuilder sentence = new StringBuilder(); 
		for(GenericParseNode g : gnp) {
			tokenizeSentences(g, sentence);
		}
				
		return sentence.toString();
		}
	}
	
	private static void tokenizeSentences(GenericParseNode n, StringBuilder s) {
		
		if(n.getContent() != null && n.getContent().length() > 0)
			s.append(n.getContent() + " ");
		
		for(GenericParseNode n1 : n.getChildren())
			tokenizeSentences(n1, s);
		
		if(n.getType().equals("sentence") && s.charAt(s.length()-2) != '*') {
			s.append("* ");
		}
	}
	
	// function to parse trees for latex
	public static String makeTree(GenericParseNode t) {
		StringBuilder sb = new StringBuilder();
		if(t.getChildren().length > 0) {
			if(!Character.isDigit(t.getType().charAt(t.getType().length()-1))) {
				sb.append("(.");
				sb.append(t.getType());
				sb.append(". ");
			}
			for(GenericParseNode t1 : t.getChildren()) {
				sb.append(makeTree(t1) + " ");
			}
			if(!Character.isDigit(t.getType().charAt(t.getType().length()-1))) {
				sb.append(")");
			}
		} else {
			sb.append("(.");
			sb.append(t.getType());
			sb.append(". `");
			sb.append(t.getType() + "=" + t.getContent());
			sb.append("')");
		}
		return sb.toString();
	}
	
	// string to work with trees
	public static void main(String args[]) {
		GenericParseNode n = GenericParseNode.parse("xu le gunta cu xagji").get(0);
		System.out.println(makeTree(n));
	}
}
