/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.parser;

import java.util.TreeMap;

public class SpanishDictionary extends AbstractDictionary {

	/** string table */
	private TreeMap<String, String> wordmap = new TreeMap<String, String>();
	
	/**
	 * Initialize wordmap.
	 */
	public SpanishDictionary() {
		wordmap.put("le", "el");
		wordmap.put("la", "un");
		wordmap.put("pe", "de");
		wordmap.put("po", "de");
		wordmap.put("noi", "incidentemente");
		wordmap.put("poi", "cual");
		wordmap.put("?", "(el?)");
		wordmap.put("ma", "(quien/que/donde...?)");
		wordmap.put("ko'a", "el");
		wordmap.put("ke'a", "el");
		wordmap.put("mi", "yo/mi");
		wordmap.put("do", "tu");
		wordmap.put("mo", "(es/esta/hace que?)");
		wordmap.put("motag", "(a/de/con/para/por...:)");
		wordmap.put("xu", "(verdad/falso?)");
		
		wordmap.put("na", "no");
		wordmap.put("a", "o");
		wordmap.put("e", "y");
		wordmap.put("o", "si-y-solamente-si");
		wordmap.put("u", "si-o-no");
	}
	
	public String getDictionaryFile() {
		return "gismu_spanish.txt";
	}

	public String[] getSeparators() {
		return new String[]{" a ", " de", " con", " por", " cerca", " porque", " utilizado por", " sobre", " para"};
	}
	
	public String getWord(String s) {
		if(wordmap.containsKey(s))
			return wordmap.get(s);
		else
			return "<" + s + ">";
	}

	public int getNounStart() {
		return 21;
	}

	public int getNounEnd() {
		return 37;
	}
	
	public int getEntryStart() {
		return 37;
	}

	public int getEntryEnd() {
		return -1;
	}

	public int getSkip() {
		return 3;
	}
}
