/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
package lojban.tests;

import junit.framework.TestCase;
import lojban.analyzer.Analysis;

public class ParseTests extends TestCase {

	public void testAbbreviatedPosession() {
		Analysis a = new Analysis("le le la maik. cribe ku karce ku klama");
		assertEquals("klama(karce[PE:cribe[PE:maik[|]|]|], -, -, -, -)", a.toString());
	}
	
	public void testIjeks() {
		Analysis a = new Analysis("la guen. prami la artr. .iseje la guen. prami la lansalot. .inasejanai la guen. melbi");
		assertEquals("or(~melbi(guen[|], -, -, -, -), ~and(prami(guen[|], lansalot[|], -, -, -), prami(guen[|], artr[|], -, -, -)))", a.toString());
	}
	
	public void testGeks() {
		Analysis a = new Analysis("ganai la guen. prami la artr. gi la guen. prami la lansalot.");
		assertEquals("or(~prami(guen[|], artr[|], -, -, -), prami(guen[|], lansalot[|], -, -, -))", a.toString());
	}
	
	public void testGeksSimple() {
		Analysis a = new Analysis("go la .artr. xabju la kamelot. gi la .artr. turni");
		assertEquals("iff(xabju(artr[|], kamelot[|], -, -, -), turni(artr[|], -, -, -, -))", a.toString());
	}
	
	public void testRelativeClauses() {
		Analysis a = new Analysis("le gerku po la maik. klama le foldi poi crino zi'e noi melbi");
		assertEquals("klama(gerku[PO:maik[|]|], foldi[POI:crino(-, -, -, -, -)|NOI:melbi(-, -, -, -, -)], -, -, -)", a.toString());
	}
	
	public void testSimpleBridi() {
		Analysis a = new Analysis("le cribe ku xabju le karci");
		assertEquals("xabju(cribe[|], karci[|], -, -, -)", a.toString());
	}
	
	public void testProsumtiAssignAnaphoricReflexive() {
		Analysis a = new Analysis("le la artr. cribe ku prami ri vo'a goi ko'a ko'a ri");
		assertEquals("prami(cribe[PE:artr[|]|], {artr[|]}, {cribe[PE:artr[|]|]}, {{cribe[PE:artr[|]|]}}, {{artr[|]}})", a.toString());
	}
	
}
