/**
 * This file modified by the Project Team (Craig Andrews,
 * Mike Itz, and Martin Meyers, 2005)
 * 
 * Portions Copyright 2001 Sun Microsystems, Inc.
 * Portions Copyright 1999-2001 Language Technologies Institute, 
 * Carnegie Mellon University.
 * All Rights Reserved.  Use is subject to license terms.
 * 
 * See the file "license.terms" for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL 
 * WARRANTIES.
 */
package textToSpeech.voice;

import java.net.URL;
import java.util.Locale;

import com.sun.speech.freetts.Age;
import com.sun.speech.freetts.Gender;
import com.sun.speech.freetts.Tokenizer;
import com.sun.speech.freetts.en.us.CMUDiphoneVoice;

/**
 * Defines an unlimited-domain diphone synthesis based voice
 */
public class LojbanDiphoneVoice extends CMUDiphoneVoice {

	/**
	 * Creates a simple voice. This is merely for backwards compatibility with
	 * versions of FreeTTS earlier than v1.2 (i.e., before the voice manager was
	 * introduced).
	 */
	public LojbanDiphoneVoice() {
		this(null, null, null, null, null, null, null, null, null);
	}

	/**
	 * Creates a simple voice
	 * 
	 * @param name
	 *            the name of the voice
	 * @param gender
	 *            the gender of the voice
	 * @param age
	 *            the age of the voice
	 * @param description
	 *            a human-readable string providing a description that can be
	 *            displayed for the users.
	 * @param locale
	 *            the locale of the voice
	 * @param domain
	 *            the domain of this voice. For example,
	 * @param organization
	 *            the organization which created the voice &quot;general&quot;,
	 *            &quot;time&quot;, or &quot;weather&quot;.
	 * @param lexicon
	 *            the lexicon to load
	 * @param database
	 *            a url to the unit database file for this voice
	 */
	public LojbanDiphoneVoice(String name, Gender gender, Age age,
			String description, Locale locale, String domain,
			String organization, LojbanLexicon lexicon, URL database) {
		super(name, gender, age, description, locale, domain, organization,
				lexicon, database);
		setRate(112.5f);
		setPitch(120F);
		setDurationStretch(1.3f);
	}

	/**
	 * Gets a tokenizer for this voice.  This voice uses the LojbanTokenizer implementation.
	 * 
	 * @return the tokenizer
	 */
	public Tokenizer getTokenizer() {
		Tokenizer tokenizer = new LojbanTokenizer();
		return tokenizer;
	}

	/**
	 * Converts this object to a string
	 * 
	 * @return a string representation of this object
	 */
	public String toString() {
		return "CMUSlowerDiphoneVoice";
	}
}