/*  Copyright 2005, Craig Andrews, Mike Itz, Martin Meyer
 
    This file is part of "Natural Language Interface Using First Order Logic"

    "Natural Language Interface Using First Order Logic" is free software;
    you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation;
    either version 2 of the License, or (at your option) any later version.

    "Natural Language Interface Using First Order Logic" is distributed
    in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * This is a lojban Tokenizer implementation.  All but four of the methods are left empty because they are unneeded in this implementation.  This implementation takes an utterance, runs it through a parser which puts a * at the end of every sentence, and separates into tokens at every space. 
 */
package textToSpeech.voice;

import java.io.Reader;

import lojban.parser.SentenceTokenizer;

import com.sun.speech.freetts.Token;
import com.sun.speech.freetts.Tokenizer;

/**
 * @author martin
 * 
 */
public class LojbanTokenizer implements Tokenizer {

	private String[] token = null;

	private int currentPosition = 0;

	/**
	 * 
	 */
	public LojbanTokenizer() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#setInputText(java.lang.String)
	 */
	public void setInputText(String utterance) {
		token = SentenceTokenizer.markSentences(utterance).split(" ");
		currentPosition = 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#setInputReader(java.io.Reader) This
	 *      implementation does nothing. We do not handle InputReaders.
	 */
	public void setInputReader(Reader arg0) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#getNextToken() We throw a
	 *      NullPointerException if token has not been set yet, else we return
	 *      the next array index and increment the counter.
	 */
	public Token getNextToken() {
		if (!hasMoreTokens()) {
			throw new ArrayIndexOutOfBoundsException("There are no more tokens in the utterance.");
		}
		
		if (token == null) {
			throw new NullPointerException(
					"The utterance to tokenize has not been set yet.");
		}
		
		Token thisToken = new Token();
		thisToken.setWhitespace(" ");
		thisToken.setPostpunctuation("");
		thisToken.setPrepunctuation("");
		thisToken.setPosition(currentPosition);
		thisToken.setLineNumber(0);
		
		if (token[currentPosition].equals("*")) {
			thisToken.setWord("");
			currentPosition++;
			return thisToken;
		}		

		thisToken.setWord(token[currentPosition]);

		// pause rule 2: words that end with a consonent are followed by a pause.
		if("bcdfgjklmnprstvxz".indexOf(token[currentPosition].charAt(token[currentPosition].length()-1)) >= 0)
			thisToken.setPostpunctuation(".");
		
		// pause rule 3: words beginning with a vowel have a pause before them.
		/*
		 * The setPrepunctuation method didn't seem to be working, so instead we set a post-pause if the *next* word needs a pre-pause.  
		 */
		currentPosition++;
		if (token.length > currentPosition
				&& "aeiouy".indexOf(token[currentPosition].charAt(0)) >= 0)
			thisToken.setPostpunctuation(".");
		return thisToken;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#hasMoreTokens()
	 */
	public boolean hasMoreTokens() {
		// we use the (length - 1) b/c the last item in the array is not usable.
		return currentPosition >= token.length - 1 ? false : true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#hasErrors()
	 */
	public boolean hasErrors() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#getErrorDescription()
	 */
	public String getErrorDescription() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#setWhitespaceSymbols(java.lang.String)
	 */
	public void setWhitespaceSymbols(String arg0) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#setSingleCharSymbols(java.lang.String)
	 */
	public void setSingleCharSymbols(String arg0) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#setPrepunctuationSymbols(java.lang.String)
	 */
	public void setPrepunctuationSymbols(String arg0) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#setPostpunctuationSymbols(java.lang.String)
	 */
	public void setPostpunctuationSymbols(String arg0) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.speech.freetts.Tokenizer#isBreak()
	 */
	public boolean isBreak() {
		return token[currentPosition-1].equals("*"); 
	}

}
